@@ -23,20 +23,23 @@ import functools
 import json
 import operator
 import os
+from shutil import copy
 
+import cf_xarray as cfxr
 import geopandas as gpd
+import h5py
 import importlib_metadata
 import julian
-import numpy as np
-import xarray as xr
 import netCDF4 as nc
+import numpy as np
 import pandas as pd
+import xarray as xr
 from shapely.geometry import Point
 from shapely.ops import transform
 
+from podaac.subsetter import dimension_cleanup as dc
 from podaac.subsetter import xarray_enhancements as xre
 
-
 GROUP_DELIM = '__'
 SERVICE_NAME = 'l2ss-py'
 
@@ -256,12 +259,17 @@ def calculate_chunks(dataset):
     -------
     dict
         The chunk dictionary, where the key is the dimension and the
-        value is 4000.
+        value is 4000 or 500 depending on how many dimensions.
     """
-    chunk_dict = {dim: 4000 for dim in dataset.dims
-                  if dataset.dims[dim] > 4000
-                  and len(dataset.dims) > 1}
-    return chunk_dict
+    if len(dataset.dims) <= 3:
+        chunk = {dim: 4000 for dim in dataset.dims
+                 if dataset.dims[dim] > 4000
+                 and len(dataset.dims) > 1}
+    else:
+        chunk = {dim: 500 for dim in dataset.dims
+                 if dataset.dims[dim] > 500}
+
+    return chunk
 
 
 def find_matching_coords(dataset, match_list):
@@ -304,7 +312,7 @@ def find_matching_coords(dataset, match_list):
     return match_coord_vars
 
 
-def get_coord_variable_names(dataset):
+def compute_coordinate_variable_names(dataset):
     """
     Given a dataset, determine the coordinate variable from a list
     of options
@@ -320,12 +328,25 @@ def get_coord_variable_names(dataset):
         Tuple of strings, where the first element is the lat coordinate
         name and the second element is the lon coordinate name
     """
+
+    dataset = xr.decode_cf(dataset)
+
+    # look for lon and lat using standard name in coordinates and axes
+    custom_criteria = {
+        "latitude": {
+            "standard_name": "latitude|projection_y_coordinate",
+        },
+        "longitude": {
+            "standard_name": "longitude|projection_x_coordinate",
+        }
+    }
+
     possible_lat_coord_names = ['lat', 'latitude', 'y']
     possible_lon_coord_names = ['lon', 'longitude', 'x']
 
     def var_is_coord(var_name, possible_coord_names):
         var_name = var_name.strip(GROUP_DELIM).split(GROUP_DELIM)[-1]
-        return var_name in possible_coord_names
+        return var_name.lower() in possible_coord_names
 
     lat_coord_names = list(filter(
         lambda var_name: var_is_coord(var_name, possible_lat_coord_names), dataset.variables))
@@ -336,6 +357,12 @@ def get_coord_variable_names(dataset):
         lat_coord_names = find_matching_coords(dataset, possible_lat_coord_names)
         lon_coord_names = find_matching_coords(dataset, possible_lon_coord_names)
 
+    # Couldn't find lon lat in data variables look in coordinates
+    if len(lat_coord_names) < 1 or len(lon_coord_names) < 1:
+        with cfxr.set_options(custom_criteria=custom_criteria):
+            lat_coord_names = dataset.cf.coordinates.get('latitude', [])
+            lon_coord_names = dataset.cf.coordinates.get('longitude', [])
+
     if len(lat_coord_names) < 1 or len(lon_coord_names) < 1:
         raise ValueError('Could not determine coordinate variables')
 
@@ -453,7 +480,7 @@ def get_spatial_bounds(dataset, lat_var_names, lon_var_names):
     return np.array([[min_lon, max_lon], [min_lat, max_lat]])
 
 
-def get_time_variable_name(dataset, lat_var):
+def compute_time_variable_name(dataset, lat_var):
     """
     Try to determine the name of the 'time' variable. This is done as
     follows:
@@ -485,15 +512,33 @@ def get_time_variable_name(dataset, lat_var):
         # per lat var)
         return time_vars[0]
 
-    for var_name in list(dataset.dims.keys()):
+    # Filter variables with 'time' in the name to avoid extra work
+    time_vars = list(filter(lambda var_name: 'time' in var_name, dataset.dims.keys()))
+
+    for var_name in time_vars:
         if "time" in var_name and dataset[var_name].squeeze().dims == lat_var.squeeze().dims:
             return var_name
     for var_name in list(dataset.data_vars.keys()):
         if "time" in var_name and dataset[var_name].squeeze().dims == lat_var.squeeze().dims:
             return var_name
+    for var_name in list(dataset.data_vars.keys()):
+        if 'time' in var_name.lower() and dataset[var_name].squeeze().dims[0] in lat_var.squeeze().dims:
+            return var_name
+
     raise ValueError('Unable to determine time variable')
 
 
+def compute_utc_name(dataset):
+    """
+    Get the name of the utc variable if it is there to determine origine time
+    """
+    for var_name in list(dataset.data_vars.keys()):
+        if 'utc' in var_name.lower() and 'time' in var_name.lower():
+            return var_name
+
+    return None
+
+
 def get_time_epoch_var(dataset, time_var_name):
     """
     Get the name of the epoch time var. This is only needed in the case
@@ -519,6 +564,13 @@ def get_time_epoch_var(dataset, time_var_name):
         epoch_var_name = time_var.attrs['comment'].split('plus')[0].strip()
     elif 'time' in dataset.variables.keys() and time_var_name != 'time':
         epoch_var_name = 'time'
+    elif any('time' in s for s in list(dataset.variables.keys())) and time_var_name != 'time':
+        for i in list(dataset.variables.keys()):
+            group_list = i.split(GROUP_DELIM)
+            if group_list[-1] == 'time':
+                epoch_var_name = i
+                break
+        return epoch_var_name
     else:
         raise ValueError('Unable to determine time variables')
 
@@ -567,7 +619,8 @@ def translate_timestamp(str_timestamp):
         '%Y-%m-%dT%H:%M:%SZ',
         '%Y-%m-%dT%H:%M:%S%Z',
         '%Y-%m-%dT%H:%M:%S.%fZ',
-        '%Y-%m-%dT%H:%M:%S.%f%Z'
+        '%Y-%m-%dT%H:%M:%S.%f%Z',
+        '%Y-%m-%d %H:%M:%S',
     ]
 
     for timestamp_format in allowed_ts_formats:
@@ -599,7 +652,7 @@ def datetime_from_mjd(dataset, time_var_name):
     time_var = dataset[time_var_name]
     if 'long_name' in time_var.attrs:
         mdj_string = time_var.attrs['long_name']
-        mjd = mdj_string[mdj_string.find("(")+1:mdj_string.find(")")].split("= ")[1]
+        mjd = mdj_string[mdj_string.find("(") + 1:mdj_string.find(")")].split("= ")[1]
         try:
             mjd_float = float(mjd)
         except ValueError:
@@ -640,13 +693,10 @@ def build_temporal_cond(min_time, max_time, dataset, time_var_name):
 
     def build_cond(str_timestamp, compare):
         timestamp = translate_timestamp(str_timestamp)
-
         if np.issubdtype(dataset[time_var_name].dtype, np.dtype(np.datetime64)):
             timestamp = pd.to_datetime(timestamp)
         if np.issubdtype(dataset[time_var_name].dtype, np.dtype(np.timedelta64)):
-
             if is_time_mjd(dataset, time_var_name):
-                # mjd when timedelta based on
                 mjd_datetime = datetime_from_mjd(dataset, time_var_name)
                 if mjd_datetime is None:
                     raise ValueError('Unable to get datetime from dataset to calculate time delta')
@@ -673,7 +723,7 @@ def build_temporal_cond(min_time, max_time, dataset, time_var_name):
     return temporal_cond
 
 
-def subset_with_bbox(dataset, lat_var_names, lon_var_names, time_var_names, bbox=None, cut=True,
+def subset_with_bbox(dataset, lat_var_names, lon_var_names, time_var_names, variables=None, bbox=None, cut=True,
                      min_time=None, max_time=None):
     """
     Subset an xarray Dataset using a spatial bounding box.
@@ -709,6 +759,8 @@ def subset_with_bbox(dataset, lat_var_names, lon_var_names, time_var_names, bbox
     if lon_bounds[0] > lon_bounds[1]:
         oper = operator.or_
 
+    lat_var_prefix = [f'{GROUP_DELIM}{GROUP_DELIM.join(x.strip(GROUP_DELIM).split(GROUP_DELIM)[:-1])}' for x in
+                      lat_var_names]
     datasets = []
     for lat_var_name, lon_var_name, time_var_name in zip(
             lat_var_names, lon_var_names, time_var_names
@@ -719,6 +771,17 @@ def subset_with_bbox(dataset, lat_var_names, lon_var_names, time_var_names, bbox
                 var for var in dataset.data_vars.keys()
                 if var.startswith(f'{GROUP_DELIM}{var_prefix}')
             ]
+            if variables:
+                group_vars.extend([
+                    var for var in dataset.data_vars.keys()
+                    if var in variables and var not in group_vars and not var.startswith(tuple(lat_var_prefix))
+                ])
+            else:
+                group_vars.extend([
+                    var for var in dataset.data_vars.keys()
+                    if var not in group_vars and not var.startswith(tuple(lat_var_prefix))
+                    ])
+
         else:
             group_vars = list(dataset.keys())
 
@@ -738,12 +801,13 @@ def subset_with_bbox(dataset, lat_var_names, lon_var_names, time_var_names, bbox
             temporal_cond,
             cut
         )
+
         datasets.append(group_dataset)
 
     return datasets
 
 
-def subset_with_shapefile(dataset, lat_var_name, lon_var_name, shapefile, cut):
+def subset_with_shapefile(dataset, lat_var_name, lon_var_name, shapefile, cut, chunks):
     """
     Subset an xarray Dataset using a shapefile
 
@@ -755,7 +819,7 @@ def subset_with_shapefile(dataset, lat_var_name, lon_var_name, shapefile, cut):
         Name of the latitude variable in the given dataset
     lon_var_name : str
         Name of the longitude variable in the given dataset
-    shapefile : np.array
+    shapefile : str
         Absolute path to the shapefile used to subset the given dataset
     cut : bool
         True if scanline should be cut.
@@ -796,8 +860,12 @@ def subset_with_shapefile(dataset, lat_var_name, lon_var_name, shapefile, cut):
         point_in_shapefile = shapefile_df.contains(point)
         return point_in_shapefile.array[0]
 
+    dask = "forbidden"
+    if chunks:
+        dask = "allowed"
+
     in_shape_vec = np.vectorize(in_shape)
-    boolean_mask = xr.apply_ufunc(in_shape_vec, dataset[lon_var_name], dataset[lat_var_name])
+    boolean_mask = xr.apply_ufunc(in_shape_vec, dataset[lon_var_name], dataset[lat_var_name], dask=dask)
     return xre.where(dataset, boolean_mask, cut)
 
 
@@ -878,7 +946,7 @@ def transform_grouped_dataset(nc_dataset, file_to_subset):
     return nc_dataset
 
 
-def recombine_grouped_datasets(datasets, output_file):
+def recombine_grouped_datasets(datasets, output_file):  # pylint: disable=too-many-branches
     """
     Given a list of xarray datasets, combine those datasets into a
     single netCDF4 Dataset and write to the disk. Each dataset has been
@@ -893,12 +961,6 @@ def recombine_grouped_datasets(datasets, output_file):
         Name of the output file to write the resulting NetCDF file to.
     """
 
-    def get_nested_group(dataset, group_path):
-        nested_group = dataset
-        for group in group_path.strip(GROUP_DELIM).split(GROUP_DELIM)[:-1]:
-            nested_group = nested_group.groups[group]
-        return nested_group
-
     base_dataset = nc.Dataset(output_file, mode='w')
 
     for dataset in datasets:
@@ -912,41 +974,11 @@ def recombine_grouped_datasets(datasets, output_file):
 
         for dim_name in list(dataset.dims.keys()):
             new_dim_name = dim_name.split(GROUP_DELIM)[-1]
-            dim_group = get_nested_group(base_dataset, dim_name)
+            dim_group = _get_nested_group(base_dataset, dim_name)
             dim_group.createDimension(new_dim_name, dataset.dims[dim_name])
 
         # Rename variables
-        for var_name in list(dataset.variables.keys()):
-            new_var_name = var_name.split(GROUP_DELIM)[-1]
-            var_group = get_nested_group(base_dataset, var_name)
-            var_dims = list(var_group.dimensions.keys())
-            variable = dataset.variables[var_name]
-            if not var_dims:
-                var_group_parent = var_group
-                # This group doesn't contain dimensions. Look at parent group to find dimensions.
-                while not var_dims:
-                    var_group_parent = var_group_parent.parent
-                    var_dims = list(var_group_parent.dimensions.keys())
-
-            if np.issubdtype(
-                    dataset.variables[var_name].dtype, np.dtype(np.datetime64)
-            ) or np.issubdtype(
-                dataset.variables[var_name].dtype, np.dtype(np.timedelta64)
-            ):
-                # Use xarray datetime encoder
-                cf_dt_coder = xr.coding.times.CFDatetimeCoder()
-                encoded_var = cf_dt_coder.encode(dataset.variables[var_name])
-                variable = encoded_var
-
-            var_group.createVariable(new_var_name, variable.dtype, var_dims)
-
-            # Copy attributes
-            var_attrs = variable.attrs
-            var_group.variables[new_var_name].setncatts(var_attrs)
-
-            # Copy data
-            var_group.variables[new_var_name].set_auto_maskandscale(False)
-            var_group.variables[new_var_name][:] = variable.data
+        _rename_variables(dataset, base_dataset)
 
     # Remove group vars from base dataset
     for var_name in list(base_dataset.variables.keys()):
@@ -964,8 +996,159 @@ def recombine_grouped_datasets(datasets, output_file):
     base_dataset.close()
 
 
-def subset(file_to_subset, bbox, output_file, variables=None,  # pylint: disable=too-many-branches
-           cut=True, shapefile=None, min_time=None, max_time=None, origin_source=None):
+def _get_nested_group(dataset, group_path):
+    nested_group = dataset
+    for group in group_path.strip(GROUP_DELIM).split(GROUP_DELIM)[:-1]:
+        nested_group = nested_group.groups[group]
+    return nested_group
+
+
+def _rename_variables(dataset, base_dataset):
+    for var_name in list(dataset.variables.keys()):
+        new_var_name = var_name.split(GROUP_DELIM)[-1]
+        var_group = _get_nested_group(base_dataset, var_name)
+        variable = dataset.variables[var_name]
+        var_dims = [x.split(GROUP_DELIM)[-1] for x in dataset.variables[var_name].dims]
+        if np.issubdtype(
+                dataset.variables[var_name].dtype, np.dtype(np.datetime64)
+        ) or np.issubdtype(
+            dataset.variables[var_name].dtype, np.dtype(np.timedelta64)
+        ):
+
+            cf_dt_coder = xr.coding.times.CFDatetimeCoder()
+            encoded_var = cf_dt_coder.encode(dataset.variables[var_name])
+            variable = encoded_var
+
+        var_attrs = variable.attrs
+        fill_value = var_attrs.get('_FillValue')
+        var_attrs.pop('_FillValue', None)
+        comp_args = {"zlib": True, "complevel": 1}
+
+        if variable.dtype == object:
+            var_group.createVariable(new_var_name, 'S1', var_dims, fill_value=fill_value, **comp_args)
+        elif variable.dtype == 'timedelta64[ns]':
+            var_group.createVariable(new_var_name, 'i4', var_dims, fill_value=fill_value, **comp_args)
+        else:
+            var_group.createVariable(new_var_name, variable.dtype, var_dims, fill_value=fill_value, **comp_args)
+
+        # Copy attributes
+        var_group.variables[new_var_name].setncatts(var_attrs)
+
+        # Copy data
+        var_group.variables[new_var_name].set_auto_maskandscale(False)
+        var_group.variables[new_var_name][:] = variable.data
+
+
+def h5file_transform(finput):
+    """
+    Transform a h5py  Dataset that has groups to an xarray compatible
+    dataset. xarray does not work with groups, so this transformation
+    will flatten the variables in the dataset and use the group path as
+    the new variable name. For example, data_01 > km > sst would become
+    'data_01__km__sst', where GROUP_DELIM is __.
+
+    Returns
+    -------
+    nc.Dataset
+        netCDF4 Dataset that does not contain groups and that has been
+        flattened.
+    """
+    data_new = h5py.File(finput, 'r+')
+    del_group_list = list(data_new.keys())
+    has_groups = bool(data_new['/'])
+
+    def walk_h5py(data_new, group):
+        # flattens h5py file
+        for key, item in data_new[group].items():
+            group_path = f'{group}{key}'
+            if isinstance(item, h5py.Dataset):
+                new_var_name = group_path.replace('/', '__')
+
+                data_new[new_var_name] = data_new[group_path]
+                del data_new[group_path]
+
+            elif isinstance(item, h5py.Group):
+                if len(list(item.keys())) == 0:
+                    new_group_name = group_path.replace('/', '__')
+                    data_new[new_group_name] = data_new[group_path]
+
+                walk_h5py(data_new, data_new[group_path].name + '/')
+
+    walk_h5py(data_new, data_new.name)
+
+    for del_group in del_group_list:
+        del data_new[del_group]
+
+    finputnc = '.'.join(finput.split('.')[:-1]) + '.nc'
+
+    data_new.close()  # close the h5py dataset
+    copy(finput, finputnc)  # copy to a nc file
+
+    nc_dataset = nc.Dataset(finputnc, mode='r')
+
+    return nc_dataset, has_groups
+
+
+def get_coordinate_variable_names(dataset, lat_var_names=None, lon_var_names=None, time_var_names=None):
+    """
+    Retrieve coordinate variables for this dataset. If coordinate
+    variables are provided, use those, Otherwise, attempt to determine
+    coordinate variables manually.
+
+    Parameters
+    ----------
+    dataset : xr.Dataset
+        xarray Dataset used to compute coordinate variables manually.
+        Only used if lat, lon, or time vars are not provided.
+    lat_var_names : list
+        List of latitude coordinate variables.
+    lon_var_names : list
+        List of longitude coordinate variables.
+    time_var_names : list
+        List of time coordinate variables.
+    """
+
+    if not lat_var_names or not lon_var_names:
+        lat_var_names, lon_var_names = compute_coordinate_variable_names(dataset)
+    if not time_var_names:
+        time_var_names = [
+            compute_time_variable_name(
+                dataset, dataset[lat_var_name]
+            ) for lat_var_name in lat_var_names
+        ]
+        time_var_names.append(compute_utc_name(dataset))
+        time_var_names = list(dict.fromkeys([x for x in time_var_names if x is not None]))  # remove Nones and any duplicates
+
+    return lat_var_names, lon_var_names, time_var_names
+
+
+def convert_to_datetime(dataset, time_vars):
+    """
+    converts the time variable to datetime if xarray doesn't decode times
+    """
+    for var in time_vars:
+        start_date = datetime.datetime.strptime("1993-01-01T00:00:00.00", "%Y-%m-%dT%H:%M:%S.%f") if any('TAI93' in str(dataset[var].attrs[attribute_name])
+                                                                                                         for attribute_name in dataset[var].attrs) else None
+
+        if np.issubdtype(dataset[var].dtype, np.dtype(float)):
+            # adjust the time values from the start date
+            if start_date:
+                dataset[var].values = [start_date + datetime.timedelta(seconds=i) for i in dataset[var].values]
+            # copy the values from the utc time variable to the time variable
+            else:
+                utc_var_name = compute_utc_name(dataset)
+                if utc_var_name:
+                    dataset[var].values = [datetime.datetime(i[0], i[1], i[2], hour=i[3], minute=i[4], second=i[5]) for i in dataset[utc_var_name].values]
+        else:
+            pass
+
+    return dataset
+
+
+def subset(file_to_subset, bbox, output_file, variables=None,
+           # pylint: disable=too-many-branches, disable=too-many-statements
+           cut=True, shapefile=None, min_time=None, max_time=None, origin_source=None,
+           lat_var_names=None, lon_var_names=None, time_var_names=None):
     """
     Subset a given NetCDF file given a bounding box
 
@@ -998,23 +1181,45 @@ def subset(file_to_subset, bbox, output_file, variables=None,  # pylint: disable
         ISO timestamp representing the upper bound of the temporal
         subset to be performed. If this value is not provided, the
         granule will not be subset temporally on the upper bound.
+    lat_var_names : list
+        List of variables that represent the latitude coordinate
+        variables for this granule. This list will only contain more
+        than one value in the case where there are multiple groups and
+        different coordinate variables for each group.
+    lon_var_names : list
+        List of variables that represent the longitude coordinate
+        variables for this granule. This list will only contain more
+        than one value in the case where there are multiple groups and
+        different coordinate variables for each group.
+    time_var_names : list
+        List of variables that represent the time coordinate
+        variables for this granule. This list will only contain more
+        than one value in the case where there are multiple groups and
+        different coordinate variables for each group.
     """
+    file_extension = file_to_subset.split('.')[-1]
 
-    # Open dataset with netCDF4 first, so we can get group info
-    nc_dataset = nc.Dataset(file_to_subset, mode='r')
+    if file_extension == 'he5':
+        nc_dataset, has_groups = h5file_transform(file_to_subset)
+    else:
+        # Open dataset with netCDF4 first, so we can get group info
+        nc_dataset = nc.Dataset(file_to_subset, mode='r')
+        has_groups = bool(nc_dataset.groups)
 
-    has_groups = bool(nc_dataset.groups)
+        # If dataset has groups, transform to work with xarray
+        if has_groups:
+            nc_dataset = transform_grouped_dataset(nc_dataset, file_to_subset)
 
-    # If dataset has groups, transform to work with xarray
-    if has_groups:
-        nc_dataset = transform_grouped_dataset(nc_dataset, file_to_subset)
+    nc_dataset = dc.remove_duplicate_dims(nc_dataset)
+
+    if variables:
+        variables = [x.replace('/', GROUP_DELIM) for x in variables]
 
     args = {
         'decode_coords': False,
         'mask_and_scale': False,
         'decode_times': False
     }
-
     if min_time or max_time:
         args['decode_times'] = True
 
@@ -1022,54 +1227,55 @@ def subset(file_to_subset, bbox, output_file, variables=None,  # pylint: disable
             xr.backends.NetCDF4DataStore(nc_dataset),
             **args
     ) as dataset:
-
-        lat_var_names, lon_var_names = get_coord_variable_names(dataset)
-        time_var_names = [
-            get_time_variable_name(
-                dataset, dataset[lat_var_name]
-            ) for lat_var_name in lat_var_names
-        ]
-        chunks_dict = calculate_chunks(dataset)
-
-        if chunks_dict:
-            dataset = dataset.chunk(chunks_dict)
-
+        lat_var_names, lon_var_names, time_var_names = get_coordinate_variable_names(
+            dataset=dataset,
+            lat_var_names=lat_var_names,
+            lon_var_names=lon_var_names,
+            time_var_names=time_var_names
+        )
+        if min_time or max_time:
+            dataset = convert_to_datetime(dataset, time_var_names)
+        chunks = calculate_chunks(dataset)
+        if chunks:
+            dataset = dataset.chunk(chunks)
         if variables:
             # Drop variables that aren't explicitly requested, except lat_var_name and
             # lon_var_name which are needed for subsetting
-            variables = [variable.upper() for variable in variables]
+            variables_upper = [variable.upper() for variable in variables]
             vars_to_drop = [
                 var_name for var_name, var in dataset.data_vars.items()
-                if var_name.upper() not in variables
+                if var_name.upper() not in variables_upper
                 and var_name not in lat_var_names
                 and var_name not in lon_var_names
                 and var_name not in time_var_names
             ]
-            dataset = dataset.drop_vars(vars_to_drop)
 
-        if bbox is not None:
+            dataset = dataset.drop_vars(vars_to_drop)
+        if shapefile:
+            datasets = [
+                subset_with_shapefile(dataset, lat_var_names[0], lon_var_names[0], shapefile, cut, chunks)
+            ]
+        elif bbox is not None:
             datasets = subset_with_bbox(
                 dataset=dataset,
                 lat_var_names=lat_var_names,
                 lon_var_names=lon_var_names,
                 time_var_names=time_var_names,
+                variables=variables,
                 bbox=bbox,
                 cut=cut,
                 min_time=min_time,
                 max_time=max_time
             )
-        elif shapefile:
-            datasets = [
-                subset_with_shapefile(dataset, lat_var_names[0], lon_var_names[0], shapefile, cut)
-            ]
+
         else:
             raise ValueError('Either bbox or shapefile must be provided')
 
         spatial_bounds = []
+
         for dataset in datasets:
             set_version_history(dataset, cut, bbox, shapefile)
             set_json_history(dataset, cut, file_to_subset, bbox, shapefile, origin_source)
-
             if has_groups:
                 spatial_bounds.append(get_spatial_bounds(
                     dataset=dataset,
@@ -1080,7 +1286,8 @@ def subset(file_to_subset, bbox, output_file, variables=None,  # pylint: disable
                 encoding = {}
                 compression = dict(zlib=True, complevel=5, _FillValue=None)
 
-                if (min_time or max_time) and any(dataset.dims.values()):
+                if (min_time or max_time) and not all(
+                        dim_size == 1 for dim_size in dataset.dims.values()):
                     encoding = {
                         var_name: {
                             'units': nc_dataset.variables[var_name].__dict__['units'],
@@ -1090,15 +1297,20 @@ def subset(file_to_subset, bbox, output_file, variables=None,  # pylint: disable
                         } for var_name in time_var_names
                         if 'units' in nc_dataset.variables[var_name].__dict__
                     }
-
                 for var in dataset.data_vars:
                     if var not in encoding:
                         encoding[var] = compression
+                    if dataset[var].dtype == 'S1' and isinstance(dataset[var].attrs.get('_FillValue'), bytes):
+                        dataset[var].attrs['_FillValue'] = dataset[var].attrs['_FillValue'].decode('UTF-8')
 
                 dataset.load().to_netcdf(output_file, 'w', encoding=encoding)
 
         if has_groups:
             recombine_grouped_datasets(datasets, output_file)
+            # Check if the spatial bounds are all 'None'. This means the
+            # subset result is empty.
+            if any(bound is None for bound in spatial_bounds):
+                return None
             return np.array([[
                 min(lon[0][0][0] for lon in zip(spatial_bounds)),
                 max(lon[0][0][1] for lon in zip(spatial_bounds))
